# praat script 
script_name$ = "NegSomat_4_onset_offset_dur_pitch_intensity.praat"
# Original script name "analyse_tier.praat" (version [2009:08:07])
# Author Daniel Hirst (email daniel.hirst@lpl.univ-aix.fr)
# Modified on [2011:03:31] by Marco Tettamanti (email tettamanti.marco@hsr.it)
version$ = "1.0 [2011:03:31]"
date$ = date$()

# Purpose: analyse a folder of Sound files and a folder of TextGrid files
# by default the folders are called "Sounds" and "TextGrids" 
# and are in a folder called Corpus which is in the same parent folder as this script
# For each interval or point the script calculates:
# - onset, offset, and duration
# and, depending on the options selected:
# - min/mean/max pitch, min/mean/max intensity, f1, f2 f3
# The results are output to the Info window and can be saved as a .txt file
# which can be read directly by a statistics package like R
#
# Adapted for sentence analysis: the script now also calculates:
# - total sound duration, total mean pitch, and total mean intensity
# The script is meant to be run after:
# "NegSomat_1_cut.praat"
# "NegSomat_2_norm.praat",
# and "NegSomat_3_shiftTextGrid.praat",
# and by default requires the presence of files with extension
# _cut_norm.wav (created by "NegSomat_2_norm.praat")
# and _cut_norm.TextGrid (created by "NegSomat_3_shiftTextGrid.praat")
# in the target folders.

#define parameters used in the script
form Define all parameters
	sentence investigator <Put your name here>
	sentence Sound_folder Corpus/Sounds
	sentence TextGrid_folder Corpus/TextGrids
	word Analysis_tier syllable
	word Sound_extension _cut_norm.wav
	word TextGrid_extension _cut_norm.TextGrid
	real Time_step 0 (= automatic)
	word Undefined_value NA
	boolean calculate_pitch yes
	boolean automatic_min_max no
	natural min_pitch 60
	natural max_pitch 750
	boolean calculate_intensity yes
	comment For formants
	boolean calculate_formants no
	natural Number_of_formants 5
	natural Maximum_formant 5500
	positive Window_length 0.05
	positive Pre_emphasis 50
	comment For point tier
	positive Analysis_window 0.1
endform


clearinfo
default_minimum_pitch = 60
default_maximum_pitch = 750

#Read in list of sound files
mySounds = Create Strings as file list... sounds
... 'sound_folder$'/*'sound_extension$'
nSounds = Get number of strings
item = 0

myTextGrids = Create Strings as file list... TextGrids
... 'textGrid_folder$'/*'textGrid_extension$'
nTextGrids = Get number of strings

if nSounds > 0 and nTextGrids > 0
	nSounds = Get number of strings
#print header of output file
	printline #File created by 'script_name$'
	... version 'version$'
	printline #Analysis carried out by ['investigator$'] 
	... on 'date$' on tier ['analysis_tier$']
	printline #
#print column names
	print  'tab$'file 'tab$'intrvl_label 'tab$'intrvl_onset 'tab$'intrvl_offset 'tab$'intrvl_duration
	if calculate_pitch
		print  'tab$'intrvl_pitch_min 'tab$'intrvl_pitch_mean 'tab$'intrvl_pitch_max
	endif
	if calculate_intensity
		print  'tab$'intrvl_intens_min  'tab$'intrvl_intens_mean  'tab$'intrvl_intens_max
	endif
	if calculate_formants
		printline  'tab$'F1 'tab$'F2 'tab$'F3
	endif
	printline

#check if TextGrid file exists for each sound
# and call treatment
	for iSound from 1 to nSounds
		select mySounds
		sound_name$ = Get string... iSound
		textGrid_name$ = sound_name$ - sound_extension$
		... + textGrid_extension$
		sound$ = sound_folder$ + "/" + sound_name$
		textGrid$ = textGrid_folder$ + "/" + textGrid_name$
		if fileReadable(textGrid$)
			call treatment
		else
			printline # Cannot find TextGrid file
			...  ['textGrid_name$']
		endif
	endfor
else
	if nSounds = 0
		printline Folder ['sound_folder$'] doesn't contain
		... any files with extension ['sound_extension$']
	elsif nTextGrids = 0
		printline Folder ['textGrid_folder$'] doesn't contain
		... any files with extension ['textGrid_extension$']
	endif
endif

#Remove file list
select mySounds
plus myTextGrids
Remove

exit

#subroutine treatment
procedure treatment
	Read from file... 'sound$'
	mySound = selected("Sound")
	name$ = selected$("Sound")
	sound_duration = Get total duration
	Read from file... 'textGrid$'
	myTextGrid = selected("TextGrid")
	textGrid_duration = Get total duration
#find number of analysis tier
	select myTextGrid
	call find_analysis_tier
	if tier
#create analysis objects
		select mySound
		printline # file : 'name$'
# -	print total sound duration
	printline # total sound duration 'sound_duration';
# - pitch
		if calculate_pitch
			if automatic_min_max
				myPitch = To Pitch... time_step
				... default_minimum_pitch default_maximum_pitch
 				call automatic_min_max_pitch
				mean_pitch = Get mean... 0 0 Hertz
			else
				myPitch = To Pitch... time_step 
				... min_pitch max_pitch
				mean_pitch = Get mean... 0 0 Hertz
			endif
			printline # minimum and maximum F0 values: min_pitch 'min_pitch:0';
			... max_pitch 'max_pitch:0'
			printline # total mean pitch  'mean_pitch'
		endif
# - intensity
		if calculate_intensity
			select mySound
			myIntensity = To Intensity... min_pitch
			... time_step Yes
				mean_intensity = Get mean... 0 0 energy
			printline # total mean intensity 'mean_intensity'
		endif
		if calculate_formants
			select mySound
			myFormants = To Formant (burg)... time_step
			... number_of_formants maximum_formant
			... window_length pre_emphasis
		endif
#Get time values of beginning and end of each interval
		select myTextGrid
		if isIntervalTier
			nIntervals = Get number of intervals... tier   
		else
			nIntervals = Get number of points... tier
		endif
		for iInterval from 1 to nIntervals
			select myTextGrid
			if isIntervalTier
				label$ = Get label of interval... tier
				... iInterval
				start = Get starting point... tier
				... iInterval
				end = Get end point... tier iInterval
			else
				label$ = Get label of point... tier
				... iInterval
				mid = Get time of point... tier iInterval
				start = mid - analysis_window/2
				if start< 0
					start = 0
				endif
				end = mid + analysis_window/2
				if end > sound_duration
					end = sound_duration
				endif
			endif
#calculate parameters for each non empty interval						
			if label$ != "" and label$ != "_" and
			... label$ != "#"
				item = item+1
# -	onset
				intrvl_onset = 1000*start
				call set_undefined intrvl_onset
				intrvl_onset$ = value$
# -	offset
				intrvl_offset = 1000*end
				call set_undefined intrvl_offset
				intrvl_offset$ = value$
# -	duration
				intrvl_duration = 1000*(end - start)
				call set_undefined intrvl_duration
				intrvl_duration$ = value$
# -	pitch
				if calculate_pitch
					select myPitch
					intrvl_pitch_min = Get minimum... start end
					... Hertz Parabolic
					call set_undefined intrvl_pitch_min
					intrvl_pitch_min$ = value$
					intrvl_pitch_mean = Get mean... start end Hertz
					call set_undefined intrvl_pitch_mean
					intrvl_pitch_mean$ = value$
					intrvl_pitch_max = Get maximum... start end
					... Hertz Parabolic
					call set_undefined intrvl_pitch_max
					intrvl_pitch_max$ = value$
				endif

# - 	intensity
				if calculate_intensity
					select myIntensity
					intensity_min = Get minimum... start
					... end Parabolic
					call set_undefined intensity_min
					intensity_min$ = value$
					intensity_mean = Get mean... start end
					... energy
					call set_undefined intensity_mean
					intensity_mean$ = value$
				intensity_max = Get maximum... start
					... end Parabolic
					call set_undefined intensity_max
					intensity_max$ = value$								
				endif

# -	formants
				if calculate_formants
					select myFormants
					f1 = Get mean... 1 start end Hertz
					call set_undefined f1
					f1$ = value$
					f2 = Get mean... 2 start end Hertz
					call set_undefined f2
					f2$ = value$
					f3 = Get mean... 3 start end Hertz
					call set_undefined f3
					f3$ = value$
				endif

#print out results
				print 'item' 'tab$''name$' 'tab$''label$'
				... 'tab$''intrvl_onset$'
				... 'tab$''intrvl_offset$'
				... 'tab$''intrvl_duration$'
				if calculate_pitch
					print 'tab$''intrvl_pitch_min$' 'tab$''intrvl_pitch_mean$'
					... 'tab$''intrvl_pitch_max$'
				endif
				if calculate_intensity
					print 'tab$''intensity_min$'
					... 'tab$''intensity_mean$'
					... 'tab$''intensity_max$'
				endif
				if calculate_formants
					print 'tab$''f1$' 'tab$''f2$'
					... 'tab$''f3$'
				endif
				printline
			endif
		endfor

#Remove objects
		if calculate_pitch
			select myPitch
			Remove
		endif
		if calculate_intensity
			select myIntensity
			Remove
		endif
		if calculate_formants
			select myFormants
			Remove
		endif
	else
#print warning if TextGrid has no analysis tier
		printline ###TextGrid ['name$'] has no tier
		... ['analysis_tier$']
	endif
#Remove Sound and TextGrid
	select mySound
	plus myTextGrid
	Remove
endproc


procedure set_undefined value
	if value = undefined
		value$ = undefined_value$
	else
		value$ = "'value:0'"
	endif
endproc

procedure automatic_min_max_pitch
	q25 = Get quantile... 0 0 0.25 Hertz
	q75 = Get quantile... 0 0 0.75 Hertz
	min_pitch = 0.75*q25
	max_pitch = 1.5*q75
	Remove
	select mySound
	myPitch = To Pitch... time_step min_pitch
	... max_pitch
endproc

procedure find_analysis_tier
	nTiers = Get number of tiers
	tier = 0
	for iTier from 1 to nTiers
		tier_name$ = Get tier name... iTier
		if tier_name$ = analysis_tier$
			tier = iTier
			isIntervalTier = Is interval tier... tier
		endif
	endfor
endproc
